@@ -83,6 +83,7 @@
 #include "rolls/sided_dice.h"
 #include "rolls/condition_checking.h"
 #include <errno.h>
+#include "pcg_basic.h"
 
 #define UNUSED(x) (void)(x)
 
@@ -101,6 +102,7 @@ int write_to_file = 0;
 char * output_file;
 
 extern int gnoll_errno;
+pcg32_random_t rng;
 
 // Registers
 
@@ -108,7 +110,13 @@ extern int gnoll_errno;
 
 int initialize(){
     if (!seeded){
-        srand((unsigned int)time(0));
+        unsigned long int tick = (unsigned long)time(0)+(unsigned long)clock();
+        pcg32_srandom_r(
+            &rng,
+            tick ^ (unsigned long int)&printf,
+            54u
+        );
+        //srand((unsigned long)time(0)+(unsigned long)clock());
         seeded = 1;
     }
     return 0;
@@ -134,7 +142,7 @@ int roll_symbolic_die(unsigned int length_of_symbolic_array){
 }
 
 
-#line 138 "y.tab.c"
+#line 146 "y.tab.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -272,11 +280,11 @@ extern int yydebug;
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 union YYSTYPE
 {
-#line 95 "src/grammar/dice.yacc"
+#line 103 "src/grammar/dice.yacc"
 
     vec values;
 
-#line 280 "y.tab.c"
+#line 288 "y.tab.c"
 
 };
 typedef union YYSTYPE YYSTYPE;
@@ -655,13 +663,13 @@ static const yytype_int8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_int16 yyrline[] =
 {
-       0,   106,   106,   108,   110,   119,   121,   126,   140,   193,
-     197,   223,   251,   281,   313,   375,   410,   439,   443,   453,
-     481,   521,   556,   576,   594,   605,   616,   628,   640,   649,
-     660,   670,   681,   687,   704,   722,   740,   757,   775,   793,
-     810,   828,   843,   862,   880,   900,   913,   929,   931,   935,
-     951,   993,  1007,  1024,  1055,  1057,  1067,  1067,  1067,  1067,
-    1067,  1067,  1070,  1077
+       0,   114,   114,   116,   118,   127,   129,   134,   148,   201,
+     205,   231,   259,   289,   321,   383,   418,   447,   451,   461,
+     489,   529,   564,   584,   602,   613,   624,   636,   648,   657,
+     668,   678,   689,   695,   712,   730,   748,   765,   783,   801,
+     818,   836,   851,   870,   888,   908,   921,   937,   939,   943,
+     959,  1001,  1015,  1032,  1063,  1065,  1075,  1075,  1075,  1075,
+    1075,  1075,  1078,  1085
 };
 #endif
 
@@ -1521,18 +1529,18 @@ yyreduce:
   switch (yyn)
     {
   case 4:
-#line 110 "src/grammar/dice.yacc"
+#line 118 "src/grammar/dice.yacc"
           {
         printf("Invalid Notation\n");
         gnoll_errno = SYNTAX_ERROR;
         YYABORT;
         yyclearin;
     }
-#line 1532 "y.tab.c"
+#line 1540 "y.tab.c"
     break;
 
   case 7:
-#line 126 "src/grammar/dice.yacc"
+#line 134 "src/grammar/dice.yacc"
                                                 {
                 
         vec key = (yyvsp[-2].values);
@@ -1545,11 +1553,11 @@ yyreduce:
             yyclearin;
         }
     }
-#line 1549 "y.tab.c"
+#line 1557 "y.tab.c"
     break;
 
   case 8:
-#line 140 "src/grammar/dice.yacc"
+#line 148 "src/grammar/dice.yacc"
                     {
 
     vec vector = (yyvsp[0].values);
@@ -1601,19 +1609,19 @@ yyreduce:
         fclose(fp);
     }
 }
-#line 1605 "y.tab.c"
-    break;
-
-  case 9:
-#line 193 "src/grammar/dice.yacc"
-                      {
-        (yyval.values) = (yyvsp[-1].values);
-    }
 #line 1613 "y.tab.c"
     break;
 
+  case 9:
+#line 201 "src/grammar/dice.yacc"
+                      {
+        (yyval.values) = (yyvsp[-1].values);
+    }
+#line 1621 "y.tab.c"
+    break;
+
   case 10:
-#line 197 "src/grammar/dice.yacc"
+#line 205 "src/grammar/dice.yacc"
                   {
         // Collapse both sides and subtract
         vec vector1;
@@ -1639,11 +1647,11 @@ yyreduce:
             (yyval.values) = new_vec;
         }
     }
-#line 1643 "y.tab.c"
+#line 1651 "y.tab.c"
     break;
 
   case 11:
-#line 223 "src/grammar/dice.yacc"
+#line 231 "src/grammar/dice.yacc"
                              {
         // Collapse both sides and subtract
         vec vector1;
@@ -1671,11 +1679,11 @@ yyreduce:
             (yyval.values) = new_vec;
         }
     }
-#line 1675 "y.tab.c"
+#line 1683 "y.tab.c"
     break;
 
   case 12:
-#line 251 "src/grammar/dice.yacc"
+#line 259 "src/grammar/dice.yacc"
                                {
         // Collapse both sides and subtract
         vec vector1;
@@ -1705,11 +1713,11 @@ yyreduce:
             (yyval.values) = new_vec;
         }
     }
-#line 1709 "y.tab.c"
+#line 1717 "y.tab.c"
     break;
 
   case 13:
-#line 281 "src/grammar/dice.yacc"
+#line 289 "src/grammar/dice.yacc"
                     {
         // Collapse both sides and subtract
         vec vector1;
@@ -1741,11 +1749,11 @@ yyreduce:
             (yyval.values) = new_vec;
         }
     }
-#line 1745 "y.tab.c"
+#line 1753 "y.tab.c"
     break;
 
   case 14:
-#line 313 "src/grammar/dice.yacc"
+#line 321 "src/grammar/dice.yacc"
                   {
         // Collapse both sides and subtract
         vec vector1;
@@ -1807,11 +1815,11 @@ yyreduce:
         }
 
     }
-#line 1811 "y.tab.c"
+#line 1819 "y.tab.c"
     break;
 
   case 15:
-#line 375 "src/grammar/dice.yacc"
+#line 383 "src/grammar/dice.yacc"
                    {
         vec vector1;
         vec vector2;
@@ -1846,11 +1854,11 @@ yyreduce:
         }
 
     }
-#line 1850 "y.tab.c"
+#line 1858 "y.tab.c"
     break;
 
   case 16:
-#line 410 "src/grammar/dice.yacc"
+#line 418 "src/grammar/dice.yacc"
                            {
         // Eltwise Negation
         vec vector;
@@ -1879,11 +1887,11 @@ yyreduce:
 
         }
     }
-#line 1883 "y.tab.c"
+#line 1891 "y.tab.c"
     break;
 
   case 18:
-#line 443 "src/grammar/dice.yacc"
+#line 451 "src/grammar/dice.yacc"
                             {
 
         vec new_vec;
@@ -1893,11 +1901,11 @@ yyreduce:
         new_vec.content[0] = (int)dice.length;
         (yyval.values) = new_vec;
     }
-#line 1897 "y.tab.c"
+#line 1905 "y.tab.c"
     break;
 
   case 19:
-#line 453 "src/grammar/dice.yacc"
+#line 461 "src/grammar/dice.yacc"
                    {
 
         vec vector;
@@ -1921,11 +1929,11 @@ yyreduce:
 
         }
     }
-#line 1925 "y.tab.c"
+#line 1933 "y.tab.c"
     break;
 
   case 20:
-#line 481 "src/grammar/dice.yacc"
+#line 489 "src/grammar/dice.yacc"
                                            {
 
         vec dice = (yyvsp[-4].values);
@@ -1966,11 +1974,11 @@ yyreduce:
             yyclearin;
         }
     }
-#line 1970 "y.tab.c"
+#line 1978 "y.tab.c"
     break;
 
   case 21:
-#line 521 "src/grammar/dice.yacc"
+#line 529 "src/grammar/dice.yacc"
                                      {
 
         vec dice = (yyvsp[-3].values);
@@ -2005,11 +2013,11 @@ yyreduce:
             yyclearin;;
         }
     }
-#line 2009 "y.tab.c"
+#line 2017 "y.tab.c"
     break;
 
   case 22:
-#line 556 "src/grammar/dice.yacc"
+#line 564 "src/grammar/dice.yacc"
                                            {
         vec new_vec;
         vec dice = (yyvsp[-3].values);
@@ -2029,11 +2037,11 @@ yyreduce:
         }
 
     }
-#line 2033 "y.tab.c"
+#line 2041 "y.tab.c"
     break;
 
   case 23:
-#line 576 "src/grammar/dice.yacc"
+#line 584 "src/grammar/dice.yacc"
                                {
         // TODO
         vec new_vec;
@@ -2051,11 +2059,11 @@ yyreduce:
             yyclearin;;
         }
     }
-#line 2055 "y.tab.c"
+#line 2063 "y.tab.c"
     break;
 
   case 24:
-#line 595 "src/grammar/dice.yacc"
+#line 603 "src/grammar/dice.yacc"
     {
         vec keep_vector = (yyvsp[0].values);
         vec new_vec;
@@ -2065,11 +2073,11 @@ yyreduce:
 
         (yyval.values) = new_vec;
     }
-#line 2069 "y.tab.c"
+#line 2077 "y.tab.c"
     break;
 
   case 25:
-#line 606 "src/grammar/dice.yacc"
+#line 614 "src/grammar/dice.yacc"
     {
         vec keep_vector = (yyvsp[0].values);
         vec new_vec;
@@ -2079,11 +2087,11 @@ yyreduce:
 
         (yyval.values) = new_vec;
     }
-#line 2083 "y.tab.c"
+#line 2091 "y.tab.c"
     break;
 
   case 26:
-#line 617 "src/grammar/dice.yacc"
+#line 625 "src/grammar/dice.yacc"
     {
         vec keep_vector;
         keep_vector = (yyvsp[0].values);
@@ -2094,11 +2102,11 @@ yyreduce:
 
         (yyval.values) = new_vec;
     }
-#line 2098 "y.tab.c"
+#line 2106 "y.tab.c"
     break;
 
   case 27:
-#line 629 "src/grammar/dice.yacc"
+#line 637 "src/grammar/dice.yacc"
     {
         vec keep_vector;
         keep_vector = (yyvsp[0].values);
@@ -2109,11 +2117,11 @@ yyreduce:
 
         (yyval.values) = new_vec;
     }
-#line 2113 "y.tab.c"
+#line 2121 "y.tab.c"
     break;
 
   case 28:
-#line 641 "src/grammar/dice.yacc"
+#line 649 "src/grammar/dice.yacc"
     {
         unsigned int num_to_hold = 1;
         vec new_vec;
@@ -2121,11 +2129,11 @@ yyreduce:
 
         (yyval.values) = new_vec;
     }
-#line 2125 "y.tab.c"
+#line 2133 "y.tab.c"
     break;
 
   case 29:
-#line 650 "src/grammar/dice.yacc"
+#line 658 "src/grammar/dice.yacc"
     {
         vec roll_vec = (yyvsp[-1].values);
         unsigned int num_to_hold = 1;
@@ -2135,11 +2143,11 @@ yyreduce:
 
         (yyval.values) = new_vec;
     }
-#line 2139 "y.tab.c"
+#line 2147 "y.tab.c"
     break;
 
   case 30:
-#line 661 "src/grammar/dice.yacc"
+#line 669 "src/grammar/dice.yacc"
     {
         unsigned int num_to_hold = 1;
 
@@ -2148,11 +2156,11 @@ yyreduce:
 
         (yyval.values) = new_vec;
     }
-#line 2152 "y.tab.c"
+#line 2160 "y.tab.c"
     break;
 
   case 31:
-#line 671 "src/grammar/dice.yacc"
+#line 679 "src/grammar/dice.yacc"
     {
         vec roll_vec = (yyvsp[-1].values);
         unsigned int num_to_hold = 1;
@@ -2162,18 +2170,18 @@ yyreduce:
 
         (yyval.values) = new_vec;
     }
-#line 2166 "y.tab.c"
+#line 2174 "y.tab.c"
     break;
 
   case 32:
-#line 682 "src/grammar/dice.yacc"
+#line 690 "src/grammar/dice.yacc"
     {
     }
-#line 2173 "y.tab.c"
+#line 2181 "y.tab.c"
     break;
 
   case 33:
-#line 688 "src/grammar/dice.yacc"
+#line 696 "src/grammar/dice.yacc"
     {
         int start_from = (yyvsp[-3].values).content[0];
 
@@ -2189,11 +2197,11 @@ yyreduce:
             start_from
         );
     }
-#line 2193 "y.tab.c"
+#line 2201 "y.tab.c"
     break;
 
   case 34:
-#line 705 "src/grammar/dice.yacc"
+#line 713 "src/grammar/dice.yacc"
     {
 
         int start_from = (yyvsp[-3].values).content[0];
@@ -2210,11 +2218,11 @@ yyreduce:
             start_from
         );
     }
-#line 2214 "y.tab.c"
+#line 2222 "y.tab.c"
     break;
 
   case 35:
-#line 723 "src/grammar/dice.yacc"
+#line 731 "src/grammar/dice.yacc"
     {
 
         int start_from = (yyvsp[-3].values).content[0];
@@ -2231,11 +2239,11 @@ yyreduce:
             start_from
         );
     }
-#line 2235 "y.tab.c"
+#line 2243 "y.tab.c"
     break;
 
   case 36:
-#line 741 "src/grammar/dice.yacc"
+#line 749 "src/grammar/dice.yacc"
     {
         int start_from = (yyvsp[-3].values).content[0];
 
@@ -2251,11 +2259,11 @@ yyreduce:
             start_from
         );
     }
-#line 2255 "y.tab.c"
+#line 2263 "y.tab.c"
     break;
 
   case 37:
-#line 758 "src/grammar/dice.yacc"
+#line 766 "src/grammar/dice.yacc"
     {
 
         int start_from = (yyvsp[-2].values).content[0];
@@ -2272,11 +2280,11 @@ yyreduce:
             start_from
         );
     }
-#line 2276 "y.tab.c"
+#line 2284 "y.tab.c"
     break;
 
   case 38:
-#line 776 "src/grammar/dice.yacc"
+#line 784 "src/grammar/dice.yacc"
     {
 
         int start_from = (yyvsp[-2].values).content[0];
@@ -2293,11 +2301,11 @@ yyreduce:
             start_from
         );
     }
-#line 2297 "y.tab.c"
+#line 2305 "y.tab.c"
     break;
 
   case 39:
-#line 794 "src/grammar/dice.yacc"
+#line 802 "src/grammar/dice.yacc"
     {
         int start_from = (yyvsp[-1].values).content[0];
 
@@ -2313,11 +2321,11 @@ yyreduce:
             start_from
         );
     }
-#line 2317 "y.tab.c"
+#line 2325 "y.tab.c"
     break;
 
   case 40:
-#line 811 "src/grammar/dice.yacc"
+#line 819 "src/grammar/dice.yacc"
     {
 
         int start_from = (yyvsp[-1].values).content[0];
@@ -2334,11 +2342,11 @@ yyreduce:
             start_from
         );
     }
-#line 2338 "y.tab.c"
+#line 2346 "y.tab.c"
     break;
 
   case 41:
-#line 829 "src/grammar/dice.yacc"
+#line 837 "src/grammar/dice.yacc"
     {
         vec dice_sides;
         initialize_vector(&dice_sides, NUMERIC, 1);
@@ -2352,11 +2360,11 @@ yyreduce:
             1
         );
     }
-#line 2356 "y.tab.c"
+#line 2364 "y.tab.c"
     break;
 
   case 42:
-#line 844 "src/grammar/dice.yacc"
+#line 852 "src/grammar/dice.yacc"
     {
 
         vec num_dice;
@@ -2374,11 +2382,11 @@ yyreduce:
             1
         );
     }
-#line 2378 "y.tab.c"
+#line 2386 "y.tab.c"
     break;
 
   case 43:
-#line 863 "src/grammar/dice.yacc"
+#line 871 "src/grammar/dice.yacc"
     {
 
         int start_from = (yyvsp[-1].values).content[0];
@@ -2395,11 +2403,11 @@ yyreduce:
             start_from
         );
     }
-#line 2399 "y.tab.c"
+#line 2407 "y.tab.c"
     break;
 
   case 44:
-#line 881 "src/grammar/dice.yacc"
+#line 889 "src/grammar/dice.yacc"
     {
         int start_from = (yyvsp[-1].values).content[0];
 
@@ -2418,11 +2426,11 @@ yyreduce:
             start_from
         );
     }
-#line 2422 "y.tab.c"
+#line 2430 "y.tab.c"
     break;
 
   case 45:
-#line 901 "src/grammar/dice.yacc"
+#line 909 "src/grammar/dice.yacc"
     {
         vec result_vec;
         initialize_vector(&result_vec, SYMBOLIC, (unsigned int)(yyvsp[-1].values).content[0]);
@@ -2434,11 +2442,11 @@ yyreduce:
         );
         (yyval.values) = result_vec;
     }
-#line 2438 "y.tab.c"
+#line 2446 "y.tab.c"
     break;
 
   case 46:
-#line 914 "src/grammar/dice.yacc"
+#line 922 "src/grammar/dice.yacc"
     {
         vec result_vec;
         vec number_of_dice;
@@ -2453,11 +2461,11 @@ yyreduce:
         );
         (yyval.values) = result_vec;
     }
-#line 2457 "y.tab.c"
+#line 2465 "y.tab.c"
     break;
 
   case 49:
-#line 936 "src/grammar/dice.yacc"
+#line 944 "src/grammar/dice.yacc"
     {
 
         // TODO: Multiple ranges
@@ -2472,11 +2480,11 @@ yyreduce:
         );
         (yyval.values) = result_vec;
     }
-#line 2476 "y.tab.c"
+#line 2484 "y.tab.c"
     break;
 
   case 50:
-#line 952 "src/grammar/dice.yacc"
+#line 960 "src/grammar/dice.yacc"
     {
         vec csd = (yyvsp[-1].values);
         vec result_vec;
@@ -2517,11 +2525,11 @@ yyreduce:
         }
         (yyval.values) = result_vec;
     }
-#line 2521 "y.tab.c"
+#line 2529 "y.tab.c"
     break;
 
   case 51:
-#line 993 "src/grammar/dice.yacc"
+#line 1001 "src/grammar/dice.yacc"
                                  {
         vec vector;
         vector = (yyvsp[0].values);
@@ -2534,11 +2542,11 @@ yyreduce:
 
         (yyval.values) = new_vector;
     }
-#line 2538 "y.tab.c"
+#line 2546 "y.tab.c"
     break;
 
   case 52:
-#line 1007 "src/grammar/dice.yacc"
+#line 1015 "src/grammar/dice.yacc"
                             {
         vec l;
         vec r;
@@ -2555,11 +2563,11 @@ yyreduce:
         );
         (yyval.values) = new_vector;
     }
-#line 2559 "y.tab.c"
+#line 2567 "y.tab.c"
     break;
 
   case 53:
-#line 1024 "src/grammar/dice.yacc"
+#line 1032 "src/grammar/dice.yacc"
                        {
         vec start = (yyvsp[-2].values);
         vec end = (yyvsp[0].values);
@@ -2590,11 +2598,11 @@ yyreduce:
         }
         (yyval.values) = new_vector;
     }
-#line 2594 "y.tab.c"
+#line 2602 "y.tab.c"
     break;
 
   case 55:
-#line 1057 "src/grammar/dice.yacc"
+#line 1065 "src/grammar/dice.yacc"
           {
         vec in = (yyvsp[0].values);
         // Max/Min int has 10 characters
@@ -2603,33 +2611,33 @@ yyreduce:
         sprintf(in.symbols[0], "%d", in.content[0]);
         (yyval.values) = in;
     }
-#line 2607 "y.tab.c"
+#line 2615 "y.tab.c"
     break;
 
   case 62:
-#line 1070 "src/grammar/dice.yacc"
+#line 1078 "src/grammar/dice.yacc"
              {
         vec new_vec;
         initialize_vector(&new_vec, NUMERIC, 1);
         new_vec.content[0] = 1;
         (yyval.values) = new_vec;
     }
-#line 2618 "y.tab.c"
+#line 2626 "y.tab.c"
     break;
 
   case 63:
-#line 1077 "src/grammar/dice.yacc"
+#line 1085 "src/grammar/dice.yacc"
                   {
         vec new_vec;
         initialize_vector(&new_vec, NUMERIC, 1);
         new_vec.content[0] = 0;
         (yyval.values) = new_vec;
     }
-#line 2629 "y.tab.c"
+#line 2637 "y.tab.c"
     break;
 
 
-#line 2633 "y.tab.c"
+#line 2641 "y.tab.c"
 
       default: break;
     }
@@ -2861,7 +2869,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 1086 "src/grammar/dice.yacc"
+#line 1094 "src/grammar/dice.yacc"
 
 /* Subroutines */
 
