@@ -4,11 +4,11 @@
 #include "dice_enums.h"
 
 typedef struct roll_params{
-    int number_of_dice;
-    int die_sides;
+    unsigned int number_of_dice;
+    unsigned int die_sides;
     EXPLOSION_TYPE explode;
     char ** symbol_pool;
     int start_value;
 } roll_params;
 
-#endif
\ No newline at end of file
+#endif
